-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved

THInGameMenuStatisticsFrame = {}
function THInGameMenuStatisticsFrame:initialize()
self.actionEventIds = {}
return true
end
function THInGameMenuStatisticsFrame:getUnitSelectionValues(unitSet)
local statisticsFrame = self:getParent()
local unitSetIndex = g_thMain:getTargetOrCurrentUnitSet(unitSet)
if statisticsFrame.playerFarm ~= nil then
local farmStatsData = g_thMain:getDataTable(statisticsFrame.playerFarm.stats)
if farmStatsData ~= nil then
local currentUnit, maxNumUnits = farmStatsData:getUnitSelectionValues(unitSetIndex)
return currentUnit, maxNumUnits, farmStatsData
end
end
return 0,0
end
function THInGameMenuStatisticsFrame:registerActionEvents()
if g_thMain.isClient then
self:unregisterActionEvents()
local contextName = Gui.INPUT_CONTEXT_MENU
local actionEventIds = self.actionEventIds
g_inputBinding:beginActionEventsModification(contextName)
local _, eventId = g_inputBinding:registerActionEvent(InputAction.UC_SELECT_UNIT, self, self.onActionSelectUnit, false, true, false, true)
g_inputBinding:setActionEventTextVisibility(eventId, false)
g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_VERY_LOW)
actionEventIds.selectUnit = eventId
g_inputBinding:endActionEventsModification()
end
end
function THInGameMenuStatisticsFrame:unregisterActionEvents()
if g_thMain.isClient then
for _, eventId in pairs(self.actionEventIds) do
g_inputBinding:removeActionEvent(eventId)
end
g_thUtils:clearTable(self.actionEventIds)
end
end
function THInGameMenuStatisticsFrame:subscribeMessageCenterEvents()
self:unsubscribeMessageCenterEvents()
g_messageCenter:subscribe(MessageType.TH_SETTING_CHANGED.CURRENTUNITSET, self.onUnitSetChanged, self)
end
function THInGameMenuStatisticsFrame:unsubscribeMessageCenterEvents()
g_messageCenter:unsubscribe(MessageType.TH_SETTING_CHANGED.CURRENTUNITSET, self)
end
function THInGameMenuStatisticsFrame:onActionSelectUnit()
local function protectedFunc()
local statisticsFrame = self:getParent()
local currentUnit, maxNumUnits, farmStatsData = self:getUnitSelectionValues()
if farmStatsData ~= nil then
if maxNumUnits > 0 then
currentUnit = math.max(currentUnit, 1) + 1
if currentUnit > maxNumUnits then
currentUnit = 1
end
else
currentUnit = 0
end
farmStatsData:setCurrentUnit(currentUnit)
statisticsFrame:updateStatistics()
end
end
g_thMain:call(protectedFunc)
end
function THInGameMenuStatisticsFrame:onUnitSetChanged()
local function protectedFunc()
local statisticsFrame = self:getParent()
statisticsFrame:updateStatistics()
end
g_thMain:call(protectedFunc)
end
function THInGameMenuStatisticsFrame:hook_onFrameOpen(superFunc, ...)
local envData = g_thMain:call("getDataTable", self, true, THInGameMenuStatisticsFrame)
local function appendFunc(...)
local protectedChunk = function()
if envData ~= nil then
envData:registerActionEvents()
envData:subscribeMessageCenterEvents()
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
function THInGameMenuStatisticsFrame:hook_onFrameClose(superFunc, ...)
local envData = g_thMain:call("getDataTable", self)
local function appendFunc(...)
local protectedChunk = function()
if envData ~= nil then
envData:unregisterActionEvents()
envData:unsubscribeMessageCenterEvents()
end
end
g_thMain:call(protectedChunk)
return ...
end
return appendFunc(superFunc(self, ...))
end
local function runScript()
g_thMain:setProtectedHook("InGameMenuStatisticsFrame", "onFrameOpen",  THInGameMenuStatisticsFrame)
g_thMain:setProtectedHook("InGameMenuStatisticsFrame", "onFrameClose", THInGameMenuStatisticsFrame)
end
g_thMain:call(runScript)